package io.xjar.boot;

import java.io.IOException;
import java.net.JarURLConnection;
import java.net.URLConnection;
import java.io.InputStream;
import java.util.Enumeration;
import java.io.Reader;
import java.io.LineNumberReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.LinkedHashSet;
import java.util.Set;
import io.xjar.key.XKey;
import io.xjar.XEncryptor;
import io.xjar.XDecryptor;
import io.xjar.XConstants;
import org.springframework.boot.loader.jar.Handler;

public class XBootURLHandler extends Handler implements XConstants
{
    private final XDecryptor xDecryptor;
    private final XEncryptor xEncryptor;
    private final XKey xKey;
    private final Set<String> indexes;
    
    public XBootURLHandler(final XDecryptor xDecryptor, final XEncryptor xEncryptor, final XKey xKey, final ClassLoader classLoader) throws Exception {
        this.xDecryptor = xDecryptor;
        this.xEncryptor = xEncryptor;
        this.xKey = xKey;
        this.indexes = new LinkedHashSet<String>();
        final Enumeration<URL> resources = classLoader.getResources("XJAR-INF/INDEXES.IDX");
        while (resources.hasMoreElements()) {
            final URL resource = resources.nextElement();
            final String url = resource.toString();
            final String classpath = url.substring(0, url.lastIndexOf("!/") + 2);
            final InputStream in = resource.openStream();
            final InputStreamReader isr = new InputStreamReader(in);
            final LineNumberReader lnr = new LineNumberReader(isr);
            String name;
            while ((name = lnr.readLine()) != null) {
                this.indexes.add(classpath + name);
            }
        }
    }
    
    protected URLConnection openConnection(final URL url) throws IOException {
        final URLConnection urlConnection = super.openConnection(url);
        return (this.indexes.contains(url.toString()) && urlConnection instanceof JarURLConnection) ? new XBootURLConnection((JarURLConnection)urlConnection, this.xDecryptor, this.xEncryptor, this.xKey) : urlConnection;
    }
}
